package com.github.polpetta.mezzotre.orm.model;

import io.ebean.annotation.ConstraintMode;
import io.ebean.annotation.DbForeignKey;
import io.ebean.annotation.Length;
import io.ebean.annotation.NotNull;
import java.util.List;
import javax.annotation.Nullable;
import javax.persistence.*;

@Entity
@Table(name = "registered_user")
public class User extends Base {

  @Id
  @Length(64)
  private final String id;

  @Column(columnDefinition = "boolean default false")
  @NotNull
  private Boolean isActive;

  @OneToOne(fetch = FetchType.LAZY, optional = true)
  @DbForeignKey(onDelete = ConstraintMode.CASCADE)
  @JoinColumn(name = "telegram_id", referencedColumnName = "id")
  private TgChat telegramChat;

  @ManyToMany(fetch = FetchType.LAZY)
  private List<Campaign> campaigns;

  @Length(256)
  @Nullable
  private String emailAddress;

  public User(String id, String emailAddress, Boolean isActive, TgChat telegramChat) {
    this.id = id;
    this.emailAddress = emailAddress;
    this.isActive = isActive;
    this.telegramChat = telegramChat;
  }

  public String getId() {
    return id;
  }

  public Boolean isActive() {
    return isActive;
  }

  public void setActive(Boolean active) {
    isActive = active;
  }

  public TgChat getTelegramChat() {
    return telegramChat;
  }

  public void setTelegramChat(TgChat telegramChat) {
    this.telegramChat = telegramChat;
  }

  @Nullable
  public String getEmailAddress() {
    return emailAddress;
  }

  public void setEmailAddress(@Nullable String emailAddress) {
    this.emailAddress = emailAddress;
  }

  public List<Campaign> getCampaigns() {
    return campaigns;
  }

  public void setCampaigns(List<Campaign> campaigns) {
    this.campaigns = campaigns;
  }
}
