-- apply changes
create table callback_query_context (
                                        id                            varchar(36) not null,
                                        entry_group                   varchar(36) not null,
                                        fields                        json not null default '{}'::json not null,
                                        entry_created                 timestamptz not null,
                                        entry_modified                timestamptz not null,
                                        constraint pk_callback_query_context primary key (id)
);

create table campaign (
                          id                            varchar(64) not null,
                          name                          varchar(256) not null,
                          description                   varchar(4096),
                          entry_created                 timestamptz not null,
                          entry_modified                timestamptz not null,
                          constraint pk_campaign primary key (id)
);

create table campaign_registered_user (
                                          campaign_id                   varchar(64) not null,
                                          registered_user_id            varchar(64) not null,
                                          constraint pk_campaign_registered_user primary key (campaign_id,registered_user_id)
);

create table telegram_chat (
                               id                            bigint generated by default as identity not null,
                               chat_context                  jsonb not null default '{}'::jsonb not null,
                               locale                        varchar(5) not null default 'en-US' not null,
                               has_help_been_shown           boolean default false not null,
                               entry_created                 timestamptz not null,
                               entry_modified                timestamptz not null,
                               constraint pk_telegram_chat primary key (id)
);

create table registered_user (
                                 id                            varchar(64) not null,
                                 is_active                     boolean default false not null,
                                 telegram_id                   bigint,
                                 email_address                 varchar(256),
                                 entry_created                 timestamptz not null,
                                 entry_modified                timestamptz not null,
                                 constraint uq_registered_user_telegram_id unique (telegram_id),
                                 constraint pk_registered_user primary key (id)
);

create table registered_user_campaign (
                                          registered_user_id            varchar(64) not null,
                                          campaign_id                   varchar(64) not null,
                                          constraint pk_registered_user_campaign primary key (registered_user_id,campaign_id)
);

-- foreign keys and indices
create index ix_campaign_registered_user_campaign on campaign_registered_user (campaign_id);
alter table campaign_registered_user add constraint fk_campaign_registered_user_campaign foreign key (campaign_id) references campaign (id) on delete restrict on update restrict;

create index ix_campaign_registered_user_registered_user on campaign_registered_user (registered_user_id);
alter table campaign_registered_user add constraint fk_campaign_registered_user_registered_user foreign key (registered_user_id) references registered_user (id) on delete restrict on update restrict;

alter table registered_user add constraint fk_registered_user_telegram_id foreign key (telegram_id) references telegram_chat (id) on delete cascade on update restrict;

create index ix_registered_user_campaign_registered_user on registered_user_campaign (registered_user_id);
alter table registered_user_campaign add constraint fk_registered_user_campaign_registered_user foreign key (registered_user_id) references registered_user (id) on delete restrict on update restrict;

create index ix_registered_user_campaign_campaign on registered_user_campaign (campaign_id);
alter table registered_user_campaign add constraint fk_registered_user_campaign_campaign foreign key (campaign_id) references campaign (id) on delete restrict on update restrict;
