package com.github.polpetta.mezzotre.orm;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

import com.github.polpetta.mezzotre.helper.Loader;
import com.github.polpetta.mezzotre.helper.TestConfig;
import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.github.polpetta.mezzotre.orm.model.query.QCallbackQueryContext;
import com.github.polpetta.types.json.CallbackQueryMetadata;
import io.ebean.Database;
import java.time.Duration;
import java.util.concurrent.TimeUnit;
import org.apache.commons.lang3.tuple.Pair;
import org.junit.jupiter.api.*;
import org.slf4j.LoggerFactory;
import org.testcontainers.containers.PostgreSQLContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

@Tag("slow")
@Tag("database")
@Tag("velocity")
@Testcontainers
class CallbackQueryContextCleanerIntegrationTest {

  @Container
  private final PostgreSQLContainer<?> postgresServer =
      new PostgreSQLContainer<>(TestConfig.POSTGRES_DOCKER_IMAGE);

  private Database database;

  private BatchBeanCleanerService batchBeanCleanerService;
  private CallbackQueryContextCleaner callbackQueryContextCleaner;

  @BeforeEach
  void setUp() throws Exception {
    database =
        Loader.connectToDatabase(Loader.loadDefaultEbeanConfigWithPostgresSettings(postgresServer));

    batchBeanCleanerService =
        new BatchBeanCleanerService(
            LoggerFactory.getLogger(BatchBeanCleanerService.class),
            Pair.of(0, TimeUnit.MILLISECONDS));
    batchBeanCleanerService.startAsync().awaitRunning(Duration.ofSeconds(10));

    callbackQueryContextCleaner =
        new CallbackQueryContextCleaner(
            batchBeanCleanerService, LoggerFactory.getLogger(CallbackQueryContextCleaner.class));
  }

  @AfterEach
  void tearDown() throws Exception {
    batchBeanCleanerService.stopAsync().awaitTerminated(Duration.ofSeconds(10));
  }

  @Test
  @Timeout(value = 1, unit = TimeUnit.MINUTES)
  void shouldDeleteByGroupId() throws Exception {
    new CallbackQueryContext("doesn't matter", "an id", new CallbackQueryMetadata()).save();

    final Integer got = callbackQueryContextCleaner.removeGroupAsync("an id").get();

    assertEquals(1, got);
    assertEquals(0, new QCallbackQueryContext().findCount());
  }

  @Test
  @Timeout(value = 1, unit = TimeUnit.MINUTES)
  void shouldDeleteById() throws Exception {
    new CallbackQueryContext("an id", "doesn't matter", new CallbackQueryMetadata()).save();

    final Integer got = callbackQueryContextCleaner.removeIdAsync("an id").get();

    assertEquals(1, got);
    assertEquals(0, new QCallbackQueryContext().findCount());
  }
}
